"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScopedLogger = exports.__ΩScopedLogger = exports.MemoryLogger = exports.ConsoleLogger = exports.Logger = exports.__ΩLoggerInterface = exports.ScopeFormatter = exports.LogLevelFormatter = exports.TimestampFormatter = exports.DefaultFormatter = exports.RemoveColorFormatter = exports.ColorFormatter = exports.__ΩLoggerFormatter = exports.__ΩLoggerTransport = exports.JSONTransport = exports.ConsoleTransport = exports.__ΩLogMessage = exports.__ΩLogData = exports.LoggerLevel = exports.__ΩLoggerLevel = void 0;
/*@ts-ignore*/
const { __ΩClassType } = require('@deepkit/core');
/*@ts-ignore*/
const { __ΩInject } = require('@deepkit/core');
function __assignType(fn, args) {
    fn.__type = args;
    return fn;
}
/*
 * Deepkit Framework
 * Copyright (C) 2021 Deepkit UG, Marc J. Schmidt
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the MIT License.
 *
 * You should have received a copy of the MIT License along with this program.
 */
const ansi_styles_1 = __importDefault(require("ansi-styles"));
const format_util_1 = __importDefault(require("format-util"));
const core_1 = require("@deepkit/core");
const injector_1 = require("@deepkit/injector");
const memory_logger_js_1 = require("./memory-logger.js");
const __ΩLoggerLevel = ['none', 'alert', 'error', 'warning', 'log', 'info', 'debug', 'debug2', 'LoggerLevel', 'PC!C"C#C$C%C&C\'C(Bw)'];
exports.__ΩLoggerLevel = __ΩLoggerLevel;
var LoggerLevel;
(function (LoggerLevel) {
    LoggerLevel[LoggerLevel["none"] = 0] = "none";
    LoggerLevel[LoggerLevel["alert"] = 1] = "alert";
    LoggerLevel[LoggerLevel["error"] = 2] = "error";
    LoggerLevel[LoggerLevel["warning"] = 3] = "warning";
    LoggerLevel[LoggerLevel["log"] = 4] = "log";
    LoggerLevel[LoggerLevel["info"] = 5] = "info";
    LoggerLevel[LoggerLevel["debug"] = 6] = "debug";
    LoggerLevel[LoggerLevel["debug2"] = 7] = "debug2";
})(LoggerLevel || (exports.LoggerLevel = LoggerLevel = {}));
const __ΩLogData = ['LogData', 'P&"LMw!y'];
exports.__ΩLogData = __ΩLogData;
const __ΩLogMessage = ['message', 'rawMessage', 'date', () => __ΩLoggerLevel, 'level', 'scope', () => __ΩLogData, 'data', 'LogMessage', 'P&4!&4"T4#n$4%&4&n\'4(Mw)y'];
exports.__ΩLogMessage = __ΩLogMessage;
class ConsoleTransport {
    constructor(withColors = true) {
        this.withColors = withColors;
    }
    write(message) {
        if (message.level === LoggerLevel.error) {
            process.stderr.write(message.message + '\n');
        }
        else {
            process.stdout.write(message.message + '\n');
        }
    }
    supportsColor() {
        return this.withColors;
    }
}
exports.ConsoleTransport = ConsoleTransport;
ConsoleTransport.__type = ['withColors', () => true, 'constructor', () => __ΩLogMessage, 'message', 'write', 'supportsColor', () => __ΩLoggerTransport, 'ConsoleTransport', 'P)2!<>""0#Pn$2%$0&P"0\'5n(x"w)'];
class JSONTransport {
    constructor() {
        this.out = process.stdout;
    }
    write(message) {
        this.out.write(JSON.stringify({
            message: message.message,
            level: message.level,
            date: message.date,
            scope: message.scope,
            data: message.data,
        }) + '\n');
    }
    supportsColor() {
        return false;
    }
}
exports.JSONTransport = JSONTransport;
JSONTransport.__type = ['v', '', 'write', 'out', function () { return process.stdout; }, () => __ΩLogMessage, 'message', 'supportsColor', () => __ΩLoggerTransport, 'JSONTransport', 'PP&2!"/"4#M3$>%Pn&2\'"0#P"0(5n)x"w*'];
const __ΩLoggerTransport = [() => __ΩLogMessage, 'message', 'write', 'supportsColor', 'LoggerTransport', 'PPn!2"$1#P)1$Mw%y'];
exports.__ΩLoggerTransport = __ΩLoggerTransport;
const __ΩLoggerFormatter = [() => __ΩLogMessage, 'message', 'format', 'LoggerFormatter', 'PPn!2"$1#Mw$y'];
exports.__ΩLoggerFormatter = __ΩLoggerFormatter;
class ColorFormatter {
    format(message) {
        if (message.level === LoggerLevel.error || message.level === LoggerLevel.alert) {
            message.message = `<red>${message.message}</red>`;
        }
        if (message.message.includes('<')) {
            message.message = message.message.replace(/<(\/)?([a-zA-Z]+)>/g, __assignType(function (a, end, color) {
                if (!ansi_styles_1.default[color])
                    return a;
                if (end === '/')
                    return ansi_styles_1.default[color].close;
                return ansi_styles_1.default[color].open;
            }, ['a', 'end', 'color', '', 'P"2!"2""2#"/$']));
        }
    }
}
exports.ColorFormatter = ColorFormatter;
ColorFormatter.colors = [
    'black',
    'red',
    'green',
    'yellow',
    'blue',
    'cyan',
    'magenta',
    'white',
    'gray',
    'grey',
];
ColorFormatter.__type = ['colors', function () { return [
        'black',
        'red',
        'green',
        'yellow',
        'blue',
        'cyan',
        'magenta',
        'white',
        'gray',
        'grey',
    ]; }, () => __ΩLogMessage, 'message', 'format', () => __ΩLoggerFormatter, 'ColorFormatter', '&F3!s>"Pn#2$$0%5n&x"w\''];
class RemoveColorFormatter {
    format(message) {
        if (message.message.includes('<')) {
            message.message = message.message.replace(/<(\/)?([a-zA-Z]+)>/g, __assignType(function (a, end, color) {
                return '';
            }, ['a', 'end', 'color', '', 'P"2!"2""2#"/$']));
        }
    }
}
exports.RemoveColorFormatter = RemoveColorFormatter;
RemoveColorFormatter.__type = [() => __ΩLogMessage, 'message', 'format', () => __ΩLoggerFormatter, 'RemoveColorFormatter', 'Pn!2"$0#5n$x"w%'];
class DefaultFormatter {
    constructor() {
        this.formatters = [new ScopeFormatter(), new LogLevelFormatter(), new TimestampFormatter()];
    }
    format(message) {
        for (const formatter of this.formatters) {
            formatter.format(message);
        }
    }
}
exports.DefaultFormatter = DefaultFormatter;
DefaultFormatter.__type = [() => __ΩLoggerFormatter, 'formatters', function () { return [new ScopeFormatter(), new LogLevelFormatter(), new TimestampFormatter()]; }, () => __ΩLogMessage, 'message', 'format', () => __ΩLoggerFormatter, 'DefaultFormatter', 'n!F3">#Pn$2%$0&5n\'x"w('];
class TimestampFormatter {
    format(message) {
        message.message = `<yellow>${new Date().toISOString()}</yellow> ${message.message}`;
    }
}
exports.TimestampFormatter = TimestampFormatter;
TimestampFormatter.__type = [() => __ΩLogMessage, 'message', 'format', () => __ΩLoggerFormatter, 'TimestampFormatter', 'Pn!2"$0#5n$x"w%'];
class LogLevelFormatter {
    format(message) {
        message.message = `[${String(LoggerLevel[message.level]).toUpperCase()}] ${message.message}`;
    }
}
exports.LogLevelFormatter = LogLevelFormatter;
LogLevelFormatter.__type = [() => __ΩLogMessage, 'message', 'format', () => __ΩLoggerFormatter, 'LogLevelFormatter', 'Pn!2"$0#5n$x"w%'];
class ScopeFormatter {
    format(message) {
        if (!message.scope)
            return;
        message.message = `(<yellow>${message.scope}</yellow>) ${message.message}`;
    }
}
exports.ScopeFormatter = ScopeFormatter;
ScopeFormatter.__type = [() => __ΩLogMessage, 'message', 'format', () => __ΩLoggerFormatter, 'ScopeFormatter', 'Pn!2"$0#5n$x"w%'];
const __ΩLoggerInterface = [() => __ΩLoggerLevel, 'level', 'name', 0, 'scoped', () => __ΩLogData, 'data', 0, () => __ΩLoggerLevel, 'is', 'message', 'alert', 'error', 'warn', 'log', 'info', 'debug', 'debug2', 'LoggerInterface', 'Pn!4"P&2#n$1%Pn&2\'n(1\'Pn)2")1*P"@2+$1,P"@2+$1-P"@2+$1.P"@2+$1/P"@2+$10P"@2+$11P"@2+$12Mw3y'];
exports.__ΩLoggerInterface = __ΩLoggerInterface;
class Logger {
    constructor(transporter = [], formatter = [], scope = '') {
        this.transporter = transporter;
        this.formatter = formatter;
        this.scope = scope;
        this.colorFormatter = new ColorFormatter;
        this.removeColorFormatter = new RemoveColorFormatter;
        /**
         * Setting a log level means only logs below or equal to this level will be handled.
         */
        this.level = LoggerLevel.info;
        this.scopeLevels = (Map.Ω = [['&'], [() => __ΩLoggerLevel, 'n!']], new Map());
        this.scopes = {};
    }
    setLevel(level) {
        this.level = LoggerLevel[level];
    }
    isScopeEnabled(name) {
        return (this.scopeLevels.get(name) ?? this.level) > LoggerLevel.none;
    }
    setScopeLevel(name, level) {
        this.scopeLevels.set(name, level);
    }
    unsetScopeLevel(name) {
        this.scopeLevels.delete(name);
    }
    getScopeLevel(name) {
        return this.scopeLevels.get(name) ?? this.level;
    }
    /**
     * Sends additional log data for the very next log/error/alert/warning/etc call.
     *
     * @example
     * ```typescript
     *
     * logger.data({user: user}).log('User logged in');
     *
     * //or
     *
     * //the given data is only used for the very next log (or error/alert/warning etc) call.
     * logger.data({user: user})
     * logger.log('User logged in');
     *
     * //at this point `data` is consumed, and for all other log calls not used anymore.
     * logger.log('another message without data');
     *
     *
     * ```
     */
    data(data) {
        this.logData = data;
        return this;
    }
    /**
     * Creates a new scoped logger. A scoped logger has the same log level, transports, and formatters as the parent logger,
     * and references them directly. This means if you change the log level on the parent logger, it will also change for all
     * scoped loggers.
     */
    scoped(name) {
        let scopedLogger = this.scopes[name];
        if (!scopedLogger) {
            const self = this;
            const scope = this.scope ? this.scope + '.' + name : name;
            scopedLogger = Object.setPrototypeOf({
                get level() {
                    return self.level;
                },
                transporter: self.transporter,
                formatter: self.formatter,
                scope,
                scopes: self.scopes,
                logData: self.logData,
                scopeLevels: self.scopeLevels,
                colorFormatter: self.colorFormatter,
                removeColorFormatter: self.removeColorFormatter,
            }, Logger.prototype);
            this.scopes[name] = scopedLogger;
        }
        return scopedLogger;
    }
    addTransport(transport) {
        this.transporter.push(transport);
    }
    setTransport(transport) {
        this.transporter = transport;
    }
    removeTransport(transport) {
        (0, core_1.arrayRemoveItem)(this.transporter, transport);
    }
    hasFormatter(formatterType) {
        for (const formatter of this.formatter) {
            if (formatter instanceof formatterType)
                return true;
        }
        return false;
    }
    hasFormatters() {
        return this.formatter.length > 0;
    }
    addFormatter(formatter) {
        this.formatter.push(formatter);
    }
    setFormatter(formatter) {
        this.formatter = formatter;
    }
    format(message) {
        for (const formatter of this.formatter) {
            formatter.format(message);
        }
    }
    is(level) {
        const scopeCheck = this.scopeLevels.size > 0 && !!this.scope ? this.scopeLevels.get(this.scope) : undefined;
        return scopeCheck !== undefined
            ? scopeCheck > LoggerLevel.none && level <= scopeCheck
            : level <= this.level;
    }
    send(messages, level, data = {}) {
        if (!this.is(level))
            return;
        if (this.logData) {
            data = this.logData;
            this.logData = undefined;
        }
        const rawMessage = format_util_1.default(...messages);
        const message = { message: rawMessage, rawMessage, level, date: new Date, scope: this.scope, data };
        this.format(message);
        for (const transport of this.transporter) {
            const formattedMessage = { ...message };
            if (transport.supportsColor()) {
                this.colorFormatter.format(formattedMessage);
                transport.write(formattedMessage);
            }
            else {
                this.removeColorFormatter.format(formattedMessage);
                transport.write(formattedMessage);
            }
        }
    }
    alert(...message) {
        this.send(message, LoggerLevel.alert);
    }
    error(...message) {
        this.send(message, LoggerLevel.error);
    }
    warn(...message) {
        this.send(message, LoggerLevel.warning);
    }
    log(...message) {
        this.send(message, LoggerLevel.log);
    }
    info(...message) {
        this.send(message, LoggerLevel.info);
    }
    debug(...message) {
        this.send(message, LoggerLevel.debug);
    }
    debug2(...message) {
        this.send(message, LoggerLevel.debug2);
    }
}
exports.Logger = Logger;
Logger.__type = ['colorFormatter', function () { return new ColorFormatter; }, 'removeColorFormatter', function () { return new RemoveColorFormatter; }, () => __ΩLoggerLevel, 'level', function () { return LoggerLevel.info; }, 'scopeLevels', function () { return (Map.Ω = [['&'], [() => __ΩLoggerLevel, 'n!']], new Map()); }, () => __ΩLogData, 'logData', () => Logger, 'scopes', function () { return {}; }, () => __ΩLoggerTransport, 'transporter', () => [], () => __ΩLoggerFormatter, 'formatter', () => [], 'scope', () => "", 'constructor', () => LoggerLevel, 'setLevel', 'name', 'isScopeEnabled', () => __ΩLoggerLevel, 'setScopeLevel', 'unsetScopeLevel', 'getScopeLevel', () => __ΩLogData, 'data', () => Logger, 'scoped', () => __ΩLoggerTransport, 'transport', 'addTransport', () => __ΩLoggerTransport, 'setTransport', () => __ΩLoggerTransport, 'removeTransport', () => __ΩClassType, () => __ΩLoggerFormatter, 'formatterType', 'hasFormatter', 'hasFormatters', () => __ΩLoggerFormatter, 'addFormatter', () => __ΩLoggerFormatter, 'setFormatter', () => __ΩLogMessage, 'message', 'format', () => __ΩLoggerLevel, 'is', 'messages', () => __ΩLoggerLevel, () => __ΩLogData, () => ({}), 'send', 'alert', 'error', 'warn', 'log', 'info', 'debug', 'debug2', () => __ΩLoggerInterface, 'Logger', '!3!<>"!3#<>$n%3&>\'!3(<>)n*3+8<P&P7,LM3-<>.Pn/F20<>1n2F23<>4&25:>6"07Pi8g2&"09P&2:)0;P&2:n<2&"0=P&2:"0>P&2:\'0?Pn@2A!0AP&2:P7B0CPnD2E"0FPnGF2E"0HPnI2E"0JPnLoK"2M"0NP)0OPnP23"0QPnRF23"0SPnT2U$0V<PnW2&)0XP"F2YnZ2&n[2A>\\"0]<P"@2U"0^P"@2U"0_P"@2U"0`P"@2U"0aP"@2U"0bP"@2U"0cP"@2U"0d5nex"wf'];
/**
 * Logger with pre-configured console transport.
 */
class ConsoleLogger extends Logger {
    constructor() {
        super([new ConsoleTransport], [new DefaultFormatter]);
    }
}
exports.ConsoleLogger = ConsoleLogger;
ConsoleLogger.__type = [() => Logger, 'constructor', 'ConsoleLogger', 'P7!P"0"5w#'];
/**
 * Logger with pre-configured memory transport.
 */
class MemoryLogger extends Logger {
    constructor(transporter = [], formatter = [], scope = '') {
        super(transporter || [], formatter, scope);
        this.memory = new memory_logger_js_1.MemoryLoggerTransport();
        if (transporter.length === 0) {
            this.transporter.push(this.memory);
        }
    }
    getOutput() {
        return this.memory.messageStrings.join('\n');
    }
    clear() {
        this.memory.messageStrings = [];
        this.memory.messages = [];
    }
}
exports.MemoryLogger = MemoryLogger;
MemoryLogger.__type = [() => Logger, 'memory', function () { return new memory_logger_js_1.MemoryLoggerTransport(); }, () => __ΩLoggerTransport, 'transporter', () => [], () => __ΩLoggerFormatter, 'formatter', () => [], 'scope', () => "", 'constructor', 'getOutput', 'clear', 'MemoryLogger', 'P7!!3">#Pn$F2%>&n\'F2(>)&2*>+"0,P&0-P"0.5w/'];
const __ΩScopedLogger = [() => __ΩInject, () => Logger, "scoped-logger", 'ScopedLogger', 'P7".#o!#w$y'];
exports.__ΩScopedLogger = __ΩScopedLogger;
exports.ScopedLogger = {
    provide: 'scoped-logger',
    transient: true,
    useFactory: __assignType(function useFactory(target, logger = new Logger) {
        return logger.scoped((0, injector_1.tokenLabel)(target.token));
    }, [() => injector_1.TransientInjectionTarget, 'target', () => Logger, 'logger', () => new Logger, 'useFactory', 'PP7!2"P7#2$>%"/&']),
};
//# sourceMappingURL=logger.js.map